<?php
/*
Plugin Name: Gravity Forms Infusionsoft Add-On
Plugin URI: http://www.seodenver.com
Description: Integrates Gravity Forms with Infusionsoft allowing form submissions to be automatically sent to your Infusionsoft account
Version: 1.0
Author: Katz Web Services, Inc.
Author URI: http://www.katzwebservices.com

------------------------------------------------------------------------
Copyright 2012 Katz Web Services, Inc.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

add_action('init',  array('GFInfusionsoft', 'init'));
register_activation_hook( __FILE__, array("GFInfusionsoft", "add_permissions"));

class GFInfusionsoft {
	
	private static $name = "Gravity Forms Infusionsoft Add-On";
    private static $path = "gravity-forms-infusionsoft/infusionsoft.php";
    private static $url = "http://www.gravityforms.com";
    private static $slug = "gravity-forms-infusionsoft";
    private static $version = "1.0";
    private static $min_gravityforms_version = "1.3.9";

    //Plugin starting point. Will load appropriate files
    public static function init(){
		global $pagenow;
		
		load_plugin_textdomain('gravity-forms-infusionsoft', FALSE, '/gravity-forms-infusionsoft/languages' );
		
		if($pagenow === 'plugins.php') {
			add_action("admin_notices", array('GFInfusionsoft', 'is_gravity_forms_installed'), 10);
		}
		
		if(self::is_gravity_forms_installed(false, false) === 0){
			add_action('after_plugin_row_' . self::$path, array('GFInfusionsoft', 'plugin_row') );
           return;
        }
		
		add_filter('plugin_action_links', array('GFInfusionsoft', 'settings_link'), 10, 2 );

        if(!self::is_gravityforms_supported()){
           return;
        }

        if(is_admin()){
            //loading translations
            load_plugin_textdomain('gravity-forms-infusionsoft', FALSE, '/gravity-forms-infusionsoft/languages' );

            add_filter("transient_update_plugins", array('GFInfusionsoft', 'check_update'));
            #add_filter("site_transient_update_plugins", array('GFInfusionsoft', 'check_update'));

            //creates a new Settings page on Gravity Forms' settings screen
            if(self::has_access("gravityforms_infusionsoft")){
                RGForms::add_settings_page("Infusionsoft", array("GFInfusionsoft", "settings_page"), self::get_base_url() . "/images/infusionsoft_wordpress_icon_32.png");
            }
        }

        //integrating with Members plugin
        if(function_exists('members_get_capabilities'))
            add_filter('members_get_capabilities', array("GFInfusionsoft", "members_get_capabilities"));

        //creates the subnav left menu
        add_filter("gform_addon_navigation", array('GFInfusionsoft', 'create_menu'));

        if(self::is_infusionsoft_page()){

            //enqueueing sack for AJAX requests
            wp_enqueue_script(array("sack"));

            //loading data lib
            require_once(self::get_base_path() . "/data.php");


            //loading Gravity Forms tooltips
            require_once(GFCommon::get_base_path() . "/tooltips.php");
            add_filter('gform_tooltips', array('GFInfusionsoft', 'tooltips'));

            //runs the setup when version changes
            self::setup();

         }
         else if(in_array(RG_CURRENT_PAGE, array("admin-ajax.php"))){

            //loading data class
            require_once(self::get_base_path() . "/data.php");

            add_action('wp_ajax_rg_update_feed_active', array('GFInfusionsoft', 'update_feed_active'));
            add_action('wp_ajax_gf_select_infusionsoft_form', array('GFInfusionsoft', 'select_infusionsoft_form'));

        }
        else{
             //handling post submission.
            add_action("gform_post_submission", array('GFInfusionsoft', 'export'), 10, 2);
        }
    }
    
    public static function is_gravity_forms_installed($asd = '', $echo = true) {
		global $pagenow, $page; $message = '';
		
		$installed = 0;
		$name = self::$name;
		if(!class_exists('RGForms')) {
			if(file_exists(WP_PLUGIN_DIR.'/gravityforms/gravityforms.php')) {
				$installed = 1;
				$message .= __(sprintf('%sGravity Forms is installed but not active. %sActivate Gravity Forms%s to use the %s plugin.%s', '<p>', '<strong><a href="'.wp_nonce_url(admin_url('plugins.php?action=activate&plugin=gravityforms/gravityforms.php'), 'activate-plugin_gravityforms/gravityforms.php').'">', '</a></strong>', $name,'</p>'), 'gravity-forms-infusionsoft');
			} else {
				$message .= <<<EOD
<p><a href="http://katz.si/gravityforms?con=banner" title="Gravity Forms Contact Form Plugin for WordPress"><img src="http://gravityforms.s3.amazonaws.com/banners/728x90.gif" alt="Gravity Forms Plugin for WordPress" width="728" height="90" style="border:none;" /></a></p>
		<h3><a href="http://katz.si/gravityforms" target="_blank">Gravity Forms</a> is required for the $name</h3>
		<p>You do not have the Gravity Forms plugin installed. <a href="http://katz.si/gravityforms">Get Gravity Forms</a> today.</p>
EOD;
			}
			
			if(!empty($message) && $echo) {
				echo '<div id="message" class="updated">'.$message.'</div>';
			}
		} else {
			return true;
		}
		return $installed;
	}
	
	public static function plugin_row(){
        if(!self::is_gravityforms_supported()){
            $message = sprintf(__("%sGravity Forms%s is required. %sPurchase it today!%s"), "<a href='http://katz.si/gravityforms'>", "</a>", "<a href='http://katz.si/gravityforms'>", "</a>");
            self::display_plugin_message($message, true);
        }
    }
    
    public static function display_plugin_message($message, $is_error = false){
    	$style = '';
        if($is_error)
            $style = 'style="background-color: #ffebe8;"';

        echo '</tr><tr class="plugin-update-tr"><td colspan="5" class="plugin-update"><div class="update-message" ' . $style . '>' . $message . '</div></td>';
    }

    public static function update_feed_active(){
        check_ajax_referer('rg_update_feed_active','rg_update_feed_active');
        $id = $_POST["feed_id"];
        $feed = GFInfusionsoftData::get_feed($id);
        GFInfusionsoftData::update_feed($id, $feed["form_id"], $_POST["is_active"], $feed["meta"]);
    }

    //--------------   Automatic upgrade ---------------------------------------------------
    
    function settings_link( $links, $file ) {
        static $this_plugin;
        if( ! $this_plugin ) $this_plugin = plugin_basename(__FILE__);
        if ( $file == $this_plugin ) {
            $settings_link = '<a href="' . admin_url( 'admin.php?page=gf_infusionsoft' ) . '" title="' . __('Select the Gravity Form you would like to integrate with Infusionsoft. Contacts generated by this form will be automatically added to your Infusionsoft account.', 'gravity-forms-infusionsoft') . '">' . __('Feeds', 'gravity-forms-infusionsoft') . '</a>';
            array_unshift( $links, $settings_link ); // before other links
            $settings_link = '<a href="' . admin_url( 'admin.php?page=gf_settings&addon=Infusionsoft' ) . '" title="' . __('Configure your Infusionsoft settings.', 'gravity-forms-infusionsoft') . '">' . __('Settings', 'gravity-forms-infusionsoft') . '</a>';
            array_unshift( $links, $settings_link ); // before other links
        }
        return $links;
    }
    
    
    //Returns true if the current page is an Feed pages. Returns false if not
    private static function is_infusionsoft_page(){
    	global $plugin_page; $current_page = '';
        $infusionsoft_pages = array("gf_infusionsoft");
        
        if(isset($_GET['page'])) {
			$current_page = trim(strtolower($_GET["page"]));
		}
		
        return (in_array($plugin_page, $infusionsoft_pages) || in_array($current_page, $infusionsoft_pages));
    }


    //Creates or updates database tables. Will only run when version changes
    private static function setup(){

        if(get_option("gf_infusionsoft_version") != self::$version)
            GFInfusionsoftData::update_table();

        update_option("gf_infusionsoft_version", self::$version);
    }

    //Adds feed tooltips to the list of tooltips
    public static function tooltips($tooltips){
        $infusionsoft_tooltips = array(
            "infusionsoft_contact_list" => "<h6>" . __("Infusionsoft List", "gravity-forms-infusionsoft") . "</h6>" . __("Select the Infusionsoft list you would like to add your contacts to.", "gravity-forms-infusionsoft"),
            "infusionsoft_gravity_form" => "<h6>" . __("Gravity Form", "gravity-forms-infusionsoft") . "</h6>" . __("Select the Gravity Form you would like to integrate with Infusionsoft. Contacts generated by this form will be automatically added to your Infusionsoft account.", "gravity-forms-infusionsoft"),
            "infusionsoft_map_fields" => "<h6>" . __("Map Fields", "gravity-forms-infusionsoft") . "</h6>" . __("Associate your Infusionsoft attributes to the appropriate Gravity Form fields by selecting.", "gravity-forms-infusionsoft"),
            "infusionsoft_optin_condition" => "<h6>" . __("Opt-In Condition", "gravity-forms-infusionsoft") . "</h6>" . __("When the opt-in condition is enabled, form submissions will only be exported to Infusionsoft when the condition is met. When disabled all form submissions will be exported.", "gravity-forms-infusionsoft"),

        );
        return array_merge($tooltips, $infusionsoft_tooltips);
    }

    //Creates Infusionsoft left nav menu under Forms
    public static function create_menu($menus){

        // Adding submenu if user has access
        $permission = self::has_access("gravityforms_infusionsoft");
        if(!empty($permission))
            $menus[] = array("name" => "gf_infusionsoft", "label" => __("Infusionsoft", "gravity-forms-infusionsoft"), "callback" =>  array("GFInfusionsoft", "infusionsoft_page"), "permission" => $permission);

        return $menus;
    }
    
    public function get_setting($key) {
	    $settings = get_option("gf_infusionsoft_settings");
	    return isset($settings[$key]) ? (empty($settings[$key]) ? false : $settings[$key]) : false;
    }
    
    public static function settings_page(){

        if(isset($_POST["uninstall"])){
            check_admin_referer("uninstall", "gf_infusionsoft_uninstall");
            self::uninstall();

            ?>
            <div class="updated fade" style="padding:20px;"><?php _e(sprintf("Gravity Forms Infusionsoft Add-On has been successfully uninstalled. It can be re-activated from the %splugins page%s.", "<a href='plugins.php'>","</a>"), "gravity-forms-infusionsoft")?></div>
            <?php
            return;
        }
        else if(isset($_POST["gf_infusionsoft_submit"])){
            check_admin_referer("update", "gf_infusionsoft_update");
           	$settings = array(
           		"key" => stripslashes($_POST["gf_infusionsoft_key"]), 
            	"appname" => stripslashes($_POST["gf_infusionsoft_appname"]), 
            	"debug" => isset($_POST["gf_infusionsoft_debug"]),
            );
            update_option("gf_infusionsoft_settings", $settings);
        }
        else{
            $settings = get_option("gf_infusionsoft_settings");
        }
        
        $settings = wp_parse_args($settings, array(
            	"key" => '',
            	"appname" => '', 
            	"debug" => false,
            ));

        $valid = self::test_api(true);

?>
        
        <form method="post" action="<?php echo remove_query_arg(array('refresh', 'retrieveListNames', '_wpnonce')); ?>" <?php if($new) { echo ' class="alignleft" style="width:60%;"'; } ?>>
            <?php wp_nonce_field("update", "gf_infusionsoft_update") ?>
            
            <h2><?php _e("Infusionsoft Account Information", "gravity-forms-infusionsoft") ?></h2>
            
            <table class="form-table">
                <tr>
                    <th scope="row"><label for="gf_infusionsoft_key"><?php _e("Infusionsoft API Key", "gravity-forms-infusionsoft"); ?></label><span class="howto"><a href="http://ug.infusionsoft.com/article/AA-00442/0/How-do-I-enable-the-Infusionsoft-API-and-generate-an-API-Key.html">Learn how to find your API key</a></th>
                    <td><input type="text" id="gf_infusionsoft_key" style="padding:5px 5px 3px;" class="code" name="gf_infusionsoft_key" size="80" value="<?php echo empty($settings["key"]) ? '' : esc_attr($settings["key"]); ?>"/></td>
                </tr>
                <tr>
                    <th scope="row"><label for="gf_infusionsoft_appname"><?php _e("Infusionsoft Account Subdomain", "gravity-forms-infusionsoft"); ?></label> </th>
                    <td><input type="text" id="gf_infusionsoft_appname" name="gf_infusionsoft_appname" size="30" value="<?php echo empty($settings["appname"]) ? '' : esc_attr($settings["appname"]); ?>"/></td>
                </tr>
                <tr>
                    <th scope="row"><label for="gf_infusionsoft_debug"><?php _e("Debug Form Submissions for Administrators", "gravity-forms-infusionsoft"); ?></label> </th>
                    <td><input type="checkbox" id="gf_infusionsoft_debug" name="gf_infusionsoft_debug" value="1" <?php checked($settings["debug"], true); ?>/></td>
                </tr>
                <tr>
                    <td colspan="2" ><input type="submit" name="gf_infusionsoft_submit" class="button-primary" value="<?php _e("Save Settings", "gravity-forms-infusionsoft") ?>" /></td>
                </tr>

            </table>
        <form action="" method="post">
            <?php wp_nonce_field("uninstall", "gf_infusionsoft_uninstall") ?>
            <?php if(GFCommon::current_user_can_any("gravityforms_infusionsoft_uninstall")){ ?>
                <div class="hr-divider"></div>

                <h3><?php _e("Uninstall Infusionsoft Add-On", "gravity-forms-infusionsoft") ?></h3>
                <div class="delete-alert"><?php _e("Warning! This operation deletes ALL Infusionsoft Feeds.", "gravity-forms-infusionsoft") ?>
                    <?php
                    $uninstall_button = '<input type="submit" name="uninstall" value="' . __("Uninstall Infusionsoft Add-On", "gravity-forms-infusionsoft") . '" class="button" onclick="return confirm(\'' . __("Warning! ALL Infusionsoft Feeds will be deleted. This cannot be undone. \'OK\' to delete, \'Cancel\' to stop", "gravity-forms-infusionsoft") . '\');"/>';
                    echo apply_filters("gform_infusionsoft_uninstall_button", $uninstall_button);
                    ?>
                </div>
            <?php } ?>
        </form>
        <?php
    }

    public static function infusionsoft_page(){
        $view = isset($_GET["view"]) ? $_GET["view"] : '';
        if($view == "edit")
            self::edit_page($_GET["id"]);
        else
            self::list_page();
    }

    //Displays the Infusionsoft feeds list page
    private static function list_page(){
        if(!self::is_gravityforms_supported()){
            die(__(sprintf("The Infusionsoft Add-On requires Gravity Forms %s. Upgrade automatically on the %sPlugin page%s.", self::$min_gravityforms_version, "<a href='plugins.php'>", "</a>"), "gravity-forms-infusionsoft"));
        }

        if(isset($_POST["action"]) && $_POST["action"] == "delete"){
            check_admin_referer("list_action", "gf_infusionsoft_list");

            $id = absint($_POST["action_argument"]);
            GFInfusionsoftData::delete_feed($id);
            ?>
            <div class="updated fade" style="padding:6px"><?php _e("Feed deleted.", "gravity-forms-infusionsoft") ?></div>
            <?php
        }
        else if (!empty($_POST["bulk_action"])){
            check_admin_referer("list_action", "gf_infusionsoft_list");
            $selected_feeds = $_POST["feed"];
            if(is_array($selected_feeds)){
                foreach($selected_feeds as $feed_id)
                    GFInfusionsoftData::delete_feed($feed_id);
            }
            ?>
            <div class="updated fade" style="padding:6px"><?php _e("Feeds deleted.", "gravity-forms-infusionsoft") ?></div>
            <?php
        }

        ?>
        <div class="wrap">
            <img alt="<?php _e("Infusionsoft Feeds", "gravity-forms-infusionsoft") ?>" src="<?php echo self::get_base_url()?>/images/infusion-logo.png" style="margin:15px 7px 0 0; display:block;" width="200" height="33" />
            <h2><?php _e("Infusionsoft Feeds", "gravity-forms-infusionsoft"); ?>
            <a class="button add-new-h2" href="admin.php?page=gf_infusionsoft&view=edit&id=0"><?php _e("Add New", "gravity-forms-infusionsoft") ?></a>
            </h2>
			
			<ul class="subsubsub">
	            <li><a href="<?php echo admin_url('admin.php?page=gf_settings&addon=Infusionsoft'); ?>">Infusionsoft Settings</a> |</li>
	            <li><a href="<?php echo admin_url('admin.php?page=gf_infusionsoft'); ?>" class="current">Infusionsoft Feeds</a></li>
	        </ul>

            <form id="feed_form" method="post">
                <?php wp_nonce_field('list_action', 'gf_infusionsoft_list') ?>
                <input type="hidden" id="action" name="action"/>
                <input type="hidden" id="action_argument" name="action_argument"/>

                <div class="tablenav">
                    <div class="alignleft actions" style="padding:8px 0 7px; 0">
                        <label class="hidden" for="bulk_action"><?php _e("Bulk action", "gravity-forms-infusionsoft") ?></label>
                        <select name="bulk_action" id="bulk_action">
                            <option value=''> <?php _e("Bulk action", "gravity-forms-infusionsoft") ?> </option>
                            <option value='delete'><?php _e("Delete", "gravity-forms-infusionsoft") ?></option>
                        </select>
                        <?php
                        echo '<input type="submit" class="button" value="' . __("Apply", "gravity-forms-infusionsoft") . '" onclick="if( jQuery(\'#bulk_action\').val() == \'delete\' && !confirm(\'' . __("Delete selected feeds? ", "gravity-forms-infusionsoft") . __("\'Cancel\' to stop, \'OK\' to delete.", "gravity-forms-infusionsoft") .'\')) { return false; } return true;"/>';
                        ?>
                    </div>
                </div>
                <table class="widefat fixed" cellspacing="0">
                    <thead>
                        <tr>
                            <th scope="col" id="cb" class="manage-column column-cb check-column" style=""><input type="checkbox" /></th>
                            <th scope="col" id="active" class="manage-column check-column"></th>
                            <th scope="col" class="manage-column"><?php _e("Form", "gravity-forms-infusionsoft") ?></th>
                        </tr>
                    </thead>

                    <tfoot>
                        <tr>
                            <th scope="col" id="cb" class="manage-column column-cb check-column" style=""><input type="checkbox" /></th>
                            <th scope="col" id="active" class="manage-column check-column"></th>
                            <th scope="col" class="manage-column"><?php _e("Form", "gravity-forms-infusionsoft") ?></th>
                        </tr>
                    </tfoot>

                    <tbody class="list:user user-list">
                        <?php

                        $settings = GFInfusionsoftData::get_feeds();
                        if(is_array($settings) && !empty($settings)){
                            foreach($settings as $setting){
                                ?>
                                <tr class='author-self status-inherit' valign="top">
                                    <th scope="row" class="check-column"><input type="checkbox" name="feed[]" value="<?php echo $setting["id"] ?>"/></th>
                                    <td><img src="<?php echo self::get_base_url() ?>/images/active<?php echo intval($setting["is_active"]) ?>.png" alt="<?php echo $setting["is_active"] ? __("Active", "gravity-forms-infusionsoft") : __("Inactive", "gravity-forms-infusionsoft");?>" title="<?php echo $setting["is_active"] ? __("Active", "gravity-forms-infusionsoft") : __("Inactive", "gravity-forms-infusionsoft");?>" onclick="ToggleActive(this, <?php echo $setting['id'] ?>); " /></td>
                                    <td class="column-title">
                                        <a href="admin.php?page=gf_infusionsoft&view=edit&id=<?php echo $setting["id"] ?>" title="<?php _e("Edit", "gravity-forms-infusionsoft") ?>"><?php echo $setting["form_title"] ?></a>
                                        <div class="row-actions">
                                            <span class="edit">
                                            <a title="Edit this setting" href="admin.php?page=gf_infusionsoft&view=edit&id=<?php echo $setting["id"] ?>" title="<?php _e("Edit", "gravity-forms-infusionsoft") ?>"><?php _e("Edit", "gravity-forms-infusionsoft") ?></a>
                                            |
                                            </span>

                                            <span class="edit">
                                            <a title="<?php _e("Delete", "gravity-forms-infusionsoft") ?>" href="javascript: if(confirm('<?php _e("Delete this feed? ", "gravity-forms-infusionsoft") ?> <?php _e("\'Cancel\' to stop, \'OK\' to delete.", "gravity-forms-infusionsoft") ?>')){ DeleteSetting(<?php echo $setting["id"] ?>);}"><?php _e("Delete", "gravity-forms-infusionsoft")?></a>

                                            </span>
                                        </div>
                                    </td>
                                </tr>
                                <?php
                            }
                        }
                        else { 
                        	$valid = self::test_api();
	                        if(!empty($valid)){
	                            ?>
	                            <tr>
	                                <td colspan="4" style="padding:20px;">
	                                    <?php _e(sprintf("You don't have any Infusionsoft feeds configured. Let's go %screate one%s!", '<a href="'.admin_url('admin.php?page=gf_infusionsoft&view=edit&id=0').'">', "</a>"), "gravity-forms-infusionsoft"); ?>
	                                </td>
	                            </tr>
	                            <?php
	                        } else{
	                            ?>
	                            <tr>
	                                <td colspan="4" style="padding:20px;">
	                                    <?php _e(sprintf("To get started, please configure your %sInfusionsoft Settings%s.", '<a href="admin.php?page=gf_settings&addon=Infusionsoft">', "</a>"), "gravity-forms-infusionsoft"); ?>
	                                </td>
	                            </tr>
	                            <?php
	                        }
	                    }
                        ?>
                    </tbody>
                </table>
            </form>
        </div>
        <script type="text/javascript">
            function DeleteSetting(id){
                jQuery("#action_argument").val(id);
                jQuery("#action").val("delete");
                jQuery("#feed_form")[0].submit();
            }
            function ToggleActive(img, feed_id){
                var is_active = img.src.indexOf("active1.png") >=0
                if(is_active){
                    img.src = img.src.replace("active1.png", "active0.png");
                    jQuery(img).attr('title','<?php _e("Inactive", "gravity-forms-infusionsoft") ?>').attr('alt', '<?php _e("Inactive", "gravity-forms-infusionsoft") ?>');
                }
                else{
                    img.src = img.src.replace("active0.png", "active1.png");
                    jQuery(img).attr('title','<?php _e("Active", "gravity-forms-infusionsoft") ?>').attr('alt', '<?php _e("Active", "gravity-forms-infusionsoft") ?>');
                }

                var mysack = new sack("<?php echo admin_url("admin-ajax.php")?>" );
                mysack.execute = 1;
                mysack.method = 'POST';
                mysack.setVar( "action", "rg_update_feed_active" );
                mysack.setVar( "rg_update_feed_active", "<?php echo wp_create_nonce("rg_update_feed_active") ?>" );
                mysack.setVar( "feed_id", feed_id );
                mysack.setVar( "is_active", is_active ? 0 : 1 );
                mysack.encVar( "cookie", document.cookie, false );
                mysack.onError = function() { alert('<?php _e("Ajax error while updating feed", "gravity-forms-infusionsoft" ) ?>' )};
                mysack.runAJAX();

                return true;
            }
        </script>
        <?php
    }

    public static function get_api(){

        if(!class_exists("Infusionsoft_Classloader"))
            require_once("Infusionsoft/infusionsoft.php");
        
        $infusionsoft_host = sprintf('%s.infusionsoft.com', self::get_setting('appname'));
		$infusionsoft_api_key = self::get_setting('key');
		
		//Below is just some magic...  Unless you are going to be communicating with more than one APP at the SAME TIME.  You can ignore it.
		Infusionsoft_AppPool::addApp(new Infusionsoft_App($infusionsoft_host, $infusionsoft_api_key, 443));
        
    }
    
    private static function test_api($echo = false) {
    	$works = true; $message = ''; $class = '';
    	$key = self::get_setting('key');
    	$appname = self::get_setting('appname');
    	
    	if(empty($appname) && empty($key)) { 
	    	$message = wpautop(__('
	    	<h3>Don\'t have an Infusionsoft account?</h3>
	    	This plugin requires an Infusionsoft account. If you have an Infusionsoft account, fill out the settings form below. Otherwise, you should sign up for an Infusionsoft account and start taking advantage of the world\'s best CRM.
	    	
	    	<a href="http://katz.si/indemo" class="button button-primary">Sign up for Infusionsoft Today!</a>
	    	'));
	    	$works = false;
	    	$class = 'updated';
    	} else if(empty($appname)) {
	    	$message = wpautop(__("Your Account Subdomain (also called \"Application Name\") is required. <label for='gf_infusionsoft_appname'><a>Enter it below</a></label>. <span class='howto'>If you access your Infusionsoft account from <span class='code' style='font-style:normal'><strong>example123</strong>.infusionsoft.com</span>, your Account Subdomain is <strong class='code' style='font-style:normal;'>example123</strong>.</span>", 'gravity-forms-infusionsoft'));
	    	$works = false;
    	} elseif(empty($key)) {
	    	$message = wpautop(__('Your API Key is required, please <label for="gf_infusionsoft_key"><a>enter your API key below</a></label>.', 'gravity-forms-infusionsoft'));
	    	$works = false;
    	} else {
	    	self::get_api();
	
			$app = Infusionsoft_AppPool::getApp();
	
			if(Infusionsoft_DataService::ping()){
				
				#$message .= '<h3>'.sprintf(__('Pinging the API Host: %s', 'gravity-forms-infusionsoft'), $app->getHostname() . ':' . $app->getPort()).'</h3>';
				#$message .= wpautop(__('Ping Succeeded!', 'gravity-forms-infusionsoft'));
				#$message .= '<h3>'.__('Testing your API Key', 'gravity-forms-infusionsoft').'</h3>';
	
				try {
					Infusionsoft_WebFormService::getMap(Infusionsoft_AppPool::getApp());
					$message .= wpautop(sprintf(__("It works: everything is communicating properly and your settings are correct. Now go %sconfigure form integration with Infusionsoft%s!", "gravity-forms-infusionsoft"), '<a href="'.admin_url('admin.php?page=gf_infusionsoft').'">', '</a>'));
				}
				catch(Exception $e){
					$works = false;
					if(strpos($e->getMessage(), "[InvalidKey]") !== FALSE){
						$message .= wpautop(sprintf(__('Your API Key is not correct, please double check your %sAPI key setting%s.', 'gravity-forms-infusionsoft'), '<label for="gf_infusionsoft_key"><a>', '</a></label>'));
					}		
					else{
						$message .= wpautop(sprintf(__('Failure to connect: %s', 'gravity-forms-infusionsoft'), $e->error));
					}
				}
			}
			else{
				$works = false;
				$message .= wpautop(__('Something is wrong. See below for details, check your settings and try again.', 'gravity-forms-infusionsoft'));
			}
	
			$exceptions = Infusionsoft_AppPool::getApp()->getExceptions();
	
			if(!empty($exceptions)) {
				$message .= '<ul class="ul-square">';
				foreach($exceptions as $exception){
					$messagetext = str_replace('[', __('Error key: [', 'gravity-forms-infusionsoft'), str_replace(']', ']<br />Error message: ', $exception->getMessage()));
					$message .= '<li style="list-style:square;">'.$messagetext.'</li>';
				}
				$message .= '</ul>';
			}
		}
		
		$class = empty($class) ? ($works ? "updated" : "error") : $class;

		if($message && $echo) {
			echo sprintf('<div id="message" class="%s">%s</div>', $class, wpautop($message));
        }
		
		return $works;
    }

    private static function edit_page(){
        ?>
        <style type="text/css">
        	label span.howto { cursor: default; }
            .infusionsoft_col_heading{padding-bottom:2px; border-bottom: 1px solid #ccc; font-weight:bold; width:50%;}
            #infusionsoft_field_list table { width: 400px; border-collapse: collapse; margin-top: 1em; }
            .infusionsoft_field_cell {padding: 6px 17px 0 0; margin-right:15px;}
            .gfield_required{color:red;}

            .feeds_validation_error{ background-color:#FFDFDF;}
            .feeds_validation_error td{ margin-top:4px; margin-bottom:6px; padding-top:6px; padding-bottom:6px; border-top:1px dotted #C89797; border-bottom:1px dotted #C89797}

            .left_header{float:left; width:200px; padding-right: 20px;}
            #infusionsoft_field_list .left_header { margin-top: 1em; }
            .margin_vertical_10{margin: 20px 0;}
            #gf_infusionsoft_list { margin-left:220px; padding-top: 1px }
            #infusionsoft_doubleoptin_warning{padding-left: 5px; padding-bottom:4px; font-size: 10px;}
        </style>
        <script type="text/javascript">
            var form = Array();
        </script>
        <div class="wrap">
            <img alt="<?php _e("Infusionsoft Feeds", "gravity-forms-infusionsoft") ?>" src="<?php echo self::get_base_url()?>/images/infusion-logo.png" style="display:block; margin:15px 7px 0 0;" width="200" height="33"/>
            <h2><?php _e("Infusionsoft Feeds", "gravity-forms-infusionsoft"); ?></h2>
            <ul class="subsubsub">
	            <li><a href="<?php echo admin_url('admin.php?page=gf_settings&addon=Infusionsoft'); ?>">Infusionsoft Settings</a> |</li>
	            <li><a href="<?php echo admin_url('admin.php?page=gf_infusionsoft'); ?>">Infusionsoft Feeds</a></li>
	        </ul>
		<div class="clear"></div>
        <?php
        //getting Infusionsoft API
        
        $api = self::get_api();
        
        //ensures valid credentials were entered in the settings page
        if(($api === false) || is_string($api)) {
            ?>
            <div class="error" id="message" style="margin-top:20px;"><?php echo wpautop(sprintf(__("We are unable to login to Infusionsoft with the provided username and API key. Please make sure they are valid in the %sSettings Page%s", "gravity-forms-infusionsoft"), "<a href='?page=gf_settings&addon=Infusionsoft'>", "</a>")); ?></div>
            <?php
            return;
        }

        //getting setting id (0 when creating a new one)
        $id = !empty($_POST["infusionsoft_setting_id"]) ? $_POST["infusionsoft_setting_id"] : absint($_GET["id"]);
        $config = empty($id) ? array("meta" => array(), "is_active" => true) : GFInfusionsoftData::get_feed($id);
		
	    //getting merge vars
        $merge_vars = array();
		
		//updating meta information
        if(isset($_POST["gf_infusionsoft_submit"])){
        	$objectType = $list_names = array();
        	$list = stripslashes($_POST["gf_infusionsoft_list"]);
			$config["meta"]["contact_object_name"] = $list;
            $config["form_id"] = absint($_POST["gf_infusionsoft_form"]);

            $is_valid = true;

            $merge_vars = self::get_fields();
            
            $field_map = array();
            foreach($merge_vars as $key => $var){
                $field_name = "infusionsoft_map_field_" . $var['tag'];
                $mapped_field = isset($_POST[$field_name]) ? stripslashes($_POST[$field_name]) : '';
                if(!empty($mapped_field)){
                    $field_map[$var['tag']] = $mapped_field;
                }
                else{
                    unset($field_map[$var['tag']]);
                    if(!empty($var['req'])) {
                    	$is_valid = false;
                    }
                }
                unset($_POST["{$field_name}"]);
            }
 			
			$config["meta"]["field_map"] = $field_map;
            $config["meta"]["optin_enabled"] = !empty($_POST["infusionsoft_optin_enable"]) ? true : false;
            $config["meta"]["optin_field_id"] = $config["meta"]["optin_enabled"] ? isset($_POST["infusionsoft_optin_field_id"]) ? $_POST["infusionsoft_optin_field_id"] : '' : "";
            $config["meta"]["optin_operator"] = $config["meta"]["optin_enabled"] ? isset($_POST["infusionsoft_optin_operator"]) ? $_POST["infusionsoft_optin_operator"] : '' : "";
            $config["meta"]["optin_value"] = $config["meta"]["optin_enabled"] ? $_POST["infusionsoft_optin_value"] : "";
			
            if($is_valid){
                $id = GFInfusionsoftData::update_feed($id, $config["form_id"], $config["is_active"], $config["meta"]);
                ?>
                <div id="message" class="updated fade" style="margin-top:10px;"><p><?php echo sprintf(__("Feed Updated. %sback to list%s", "gravity-forms-infusionsoft"), "<a href='?page=gf_infusionsoft'>", "</a>") ?></p>
	                <input type="hidden" name="infusionsoft_setting_id" value="<?php echo $id ?>"/>
                </div>
                <?php
            }
            else{
                ?>
                <div class="error" style="padding:6px"><?php echo __("Feed could not be updated. Please enter all required information below.", "gravity-forms-infusionsoft") ?></div>
                <?php
            }
            
        }
        
		if(!function_exists('gform_tooltip')) {
			require_once(GFCommon::get_base_path() . "/tooltips.php");
		}
		
?>
        <form method="post" action="<?php echo remove_query_arg('refresh'); ?>">
            <input type="hidden" name="infusionsoft_setting_id" value="<?php echo $id ?>"/>

            <div id="infusionsoft_form_container" valign="top" class="margin_vertical_10">
            	
            	<h2><?php _e('1. Select the form to tap into.', "gravity-forms-infusionsoft"); ?></h2>
            	<?php
            	$forms = RGFormsModel::get_forms();
		
				if(isset($config["form_id"])) {
					foreach($forms as $form) {
						if($form->id == $config["form_id"]) {
							echo '<h3 style="margin:0; padding:0 0 1em 1.75em; font-weight:normal;">'.sprintf(__('(Currently linked with %s)', "gravity-forms-infusionsoft"), $form->title).'</h3>';
						}
					}
				}
				
				?>
                <label for="gf_infusionsoft_form" class="left_header"><?php _e("Gravity Form", "gravity-forms-infusionsoft"); ?> <?php gform_tooltip("infusionsoft_gravity_form") ?></label>

                <select id="gf_infusionsoft_form" name="gf_infusionsoft_form" onchange="SelectForm(jQuery(this).val());">
                <option value=""><?php _e("Select a form", "gravity-forms-infusionsoft"); ?> </option>
                <?php
                
                foreach($forms as $form){
                    $selected = absint($form->id) == $config["form_id"] ? "selected='selected'" : "";
                    ?>
                    <option value="<?php echo absint($form->id) ?>"  <?php echo $selected ?>><?php echo esc_html($form->title) ?></option>
                    <?php
                }
                ?>
                </select>
                &nbsp;&nbsp;
                <img src="<?php echo GFInfusionsoft::get_base_url() ?>/images/loading.gif" id="infusionsoft_wait" style="display: none;"/>
            </div>
            
            <div class="clear"></div>
            <div id="infusionsoft_field_group" valign="top" <?php echo empty($config["form_id"]) ? "style='display:none;'" : "" ?>>
            	<div id="infusionsoft_field_container" valign="top" class="margin_vertical_10" >
                	<h2><?php _e('2. Map form fields to Infusionsoft fields.', "gravity-forms-infusionsoft"); ?></h2>
                	<h3 class="description"><?php _e('About field mapping:', "gravity-forms-infusionsoft"); ?></h2>
                	<label for="infusionsoft_fields" class="left_header"><?php _e("Standard Fields", "gravity-forms-infusionsoft"); ?> <?php gform_tooltip("infusionsoft_map_fields") ?></label>
                    <div id="infusionsoft_field_list">
                    <?php
                    
                    if(!empty($config["form_id"])){

                        //getting list of all Infusionsoft merge variables for the selected contact list
                        if(empty($merge_vars))
                            $merge_vars = self::get_fields($config['meta']['contact_object_name']);
                        
                        //getting field map UI
                        echo self::get_field_mapping($config, $config["form_id"], $merge_vars);

                        //getting list of selection fields to be used by the optin
                        $form_meta = RGFormsModel::get_form_meta($config["form_id"]);
                        $selection_fields = GFCommon::get_selection_fields($form_meta, $config["meta"]["optin_field_id"]);
                    }
                    
                    ?>
                    </div>
                    <div class="clear"></div>
                </div>
				
                <div id="infusionsoft_optin_container" valign="top" class="margin_vertical_10">
                    <label for="infusionsoft_optin" class="left_header"><?php _e("Opt-In Condition", "gravity-forms-infusionsoft"); ?> <?php gform_tooltip("infusionsoft_optin_condition") ?></label>
                    <div id="infusionsoft_optin">
                        <table>
                            <tr>
                                <td>
                                    <input type="checkbox" id="infusionsoft_optin_enable" name="infusionsoft_optin_enable" value="1" onclick="if(this.checked){jQuery('#infusionsoft_optin_condition_field_container').show('slow');} else{jQuery('#infusionsoft_optin_condition_field_container').hide('slow');}" <?php echo !empty($config["meta"]["optin_enabled"]) ? "checked='checked'" : ""?>/>
                                    <label for="infusionsoft_optin_enable"><?php _e("Enable", "gravity-forms-infusionsoft"); ?></label>
                                </td>
                            </tr>
                            <tr>
                                <td>
                                    <div id="infusionsoft_optin_condition_field_container" <?php echo empty($config["meta"]["optin_enabled"]) ? "style='display:none'" : ""?>>
                                        <div id="infusionsoft_optin_condition_fields" <?php echo empty($selection_fields) ? "style='display:none'" : ""?>>
                                            <?php _e("Export to Infusionsoft if ", "gravity-forms-infusionsoft") ?>

                                            <select id="infusionsoft_optin_field_id" name="infusionsoft_optin_field_id" class='optin_select' onchange='jQuery("#infusionsoft_optin_value").html(GetFieldValues(jQuery(this).val(), "", 20));'><?php echo $selection_fields ?></select>
                                            <select id="infusionsoft_optin_operator" name="infusionsoft_optin_operator" />
                                                <option value="is" <?php echo (isset($config["meta"]["optin_operator"]) && $config["meta"]["optin_operator"] == "is") ? "selected='selected'" : "" ?>><?php _e("is", "gravity-forms-infusionsoft") ?></option>
                                                <option value="isnot" <?php echo (isset($config["meta"]["optin_operator"]) && $config["meta"]["optin_operator"] == "isnot") ? "selected='selected'" : "" ?>><?php _e("is not", "gravity-forms-infusionsoft") ?></option>
                                            </select>
                                            <select id="infusionsoft_optin_value" name="infusionsoft_optin_value" class='optin_select'>
                                            </select>

                                        </div>
                                        <div id="infusionsoft_optin_condition_message" <?php echo !empty($selection_fields) ? "style='display:none'" : ""?>>
                                            <?php _e("To create an Opt-In condition, your form must have a drop down, checkbox or multiple choice field.", "gravityform") ?>
                                        </div>
                                    </div>
                                </td>
                            </tr>
                        </table>
                    </div>

                    <script type="text/javascript">
                        <?php
                        if(!empty($config["form_id"])){
                            ?>
                            //creating Javascript form object
                            form = <?php echo GFCommon::json_encode($form_meta)?> ;

                            //initializing drop downs
                            jQuery(document).ready(function(){
                                var selectedField = "<?php echo str_replace('"', '\"', $config["meta"]["optin_field_id"])?>";
                                var selectedValue = "<?php echo str_replace('"', '\"', $config["meta"]["optin_value"])?>";
                                SetOptin(selectedField, selectedValue);
                            });
                        <?php
                        }
                        ?>
                    </script>
                </div>

                <div id="infusionsoft_submit_container" class="margin_vertical_10">
                    <input type="submit" name="gf_infusionsoft_submit" value="<?php echo empty($id) ? __("Save Feed", "gravity-forms-infusionsoft") : __("Update Feed", "gravity-forms-infusionsoft"); ?>" class="button-primary"/>
                </div>
            </div>
        </form>
        </div>
		
<script type="text/javascript">
	jQuery(document).ready(function($) { 
			
	<?php if(isset($_REQUEST['id'])) { ?>
		$('#infusionsoft_field_list').live('load', function() {
			$('.infusionsoft_field_cell select').each(function() {
				var $select = $(this);
				if($().prop) {
					var label = $.trim($('label[for='+$(this).prop('name')+']').text());
				} else {
					var label = $.trim($('label[for='+$(this).attr('name')+']').text());
				}
				label = label.replace(' *', '');
				
				if($select.val() === '') {
					$('option', $select).each(function() {
						
						if($(this).text() === label) {
							if($().prop) {
								$(this).prop('selected', true);
							} else {
								$(this).attr('selected', true);
							}
						}
					});
				}
			});
		});
	<?php } ?>
	});
		</script>
		<script type="text/javascript">

		<?php if(empty($config["form_id"])){ ?>
			jQuery(document).ready(function() { 
				
				SelectForm(jQuery('#gf_infusionsoft_form').val());
				
			});
		<?php } ?>

            function SelectForm(formId){
	            		
                if(!formId){
                    jQuery("#infusionsoft_field_group").slideUp();
                    return;
                }

                jQuery("#infusionsoft_wait").show();
                jQuery("#infusionsoft_field_group").slideUp();

                var mysack = new sack("<?php bloginfo( 'wpurl' ); ?>/wp-admin/admin-ajax.php" );
                mysack.execute = 1;
                mysack.method = 'POST';
                mysack.setVar( "action", "gf_select_infusionsoft_form" );
                mysack.setVar( "gf_select_infusionsoft_form", "<?php echo wp_create_nonce("gf_select_infusionsoft_form") ?>" );
                mysack.setVar( "form_id", formId);
                mysack.encVar( "cookie", document.cookie, false );
                mysack.onError = function() {jQuery("#infusionsoft_wait").hide(); alert('<?php _e("Ajax error while selecting a form", "gravity-forms-infusionsoft") ?>' )};
                mysack.runAJAX();
                return true;
            }

            function SetOptin(selectedField, selectedValue){

                //load form fields
                jQuery("#infusionsoft_optin_field_id").html(GetSelectableFields(selectedField, 20));
                var optinConditionField = jQuery("#infusionsoft_optin_field_id").val();

                if(optinConditionField){
                    jQuery("#infusionsoft_optin_condition_message").hide();
                    jQuery("#infusionsoft_optin_condition_fields").show();
                    jQuery("#infusionsoft_optin_value").html(GetFieldValues(optinConditionField, selectedValue, 20));
                }
                else{
                    jQuery("#infusionsoft_optin_condition_message").show();
                    jQuery("#infusionsoft_optin_condition_fields").hide();
                }
            }

            function EndSelectForm(fieldList, form_meta){
                //setting global form object
                form = form_meta;

                if(fieldList){

                    SetOptin("","");

                    jQuery("#infusionsoft_field_list").html(fieldList);
                    jQuery("#infusionsoft_field_group").slideDown();
					jQuery('#infusionsoft_field_list').trigger('load');
                }
                else{
                    jQuery("#infusionsoft_field_group").slideUp();
                    jQuery("#infusionsoft_field_list").html("");
                }
                jQuery("#infusionsoft_wait").hide();
            }

            function GetFieldValues(fieldId, selectedValue, labelMaxCharacters){
                if(!fieldId)
                    return "";

                var str = "";
                var field = GetFieldById(fieldId);
                if(!field || !field.choices)
                    return "";

                var isAnySelected = false;

                for(var i=0; i<field.choices.length; i++){
                    var fieldValue = field.choices[i].value ? field.choices[i].value : field.choices[i].text;
                    var isSelected = fieldValue == selectedValue;
                    var selected = isSelected ? "selected='selected'" : "";
                    if(isSelected)
                        isAnySelected = true;

                    str += "<option value='" + fieldValue.replace("'", "&#039;") + "' " + selected + ">" + TruncateMiddle(field.choices[i].text, labelMaxCharacters) + "</option>";
                }

                if(!isAnySelected && selectedValue){
                    str += "<option value='" + selectedValue.replace("'", "&#039;") + "' selected='selected'>" + TruncateMiddle(selectedValue, labelMaxCharacters) + "</option>";
                }

                return str;
            }

            function GetFieldById(fieldId){
                for(var i=0; i<form.fields.length; i++){
                    if(form.fields[i].id == fieldId)
                        return form.fields[i];
                }
                return null;
            }

            function TruncateMiddle(text, maxCharacters){
                if(text.length <= maxCharacters)
                    return text;
                var middle = parseInt(maxCharacters / 2);
                return text.substr(0, middle) + "..." + text.substr(text.length - middle, middle);
            }

            function GetSelectableFields(selectedFieldId, labelMaxCharacters){
                var str = "";
                var inputType;
                for(var i=0; i<form.fields.length; i++){
                    fieldLabel = form.fields[i].adminLabel ? form.fields[i].adminLabel : form.fields[i].label;
                    inputType = form.fields[i].inputType ? form.fields[i].inputType : form.fields[i].type;
                    if(inputType == "checkbox" || inputType == "radio" || inputType == "select"){
                        var selected = form.fields[i].id == selectedFieldId ? "selected='selected'" : "";
                        str += "<option value='" + form.fields[i].id + "' " + selected + ">" + TruncateMiddle(fieldLabel, labelMaxCharacters) + "</option>";
                    }
                }
                return str;
            }

        </script>
		
        <?php

    }

    public static function add_permissions(){
        global $wp_roles;
        $wp_roles->add_cap("administrator", "gravityforms_infusionsoft");
        $wp_roles->add_cap("administrator", "gravityforms_infusionsoft_uninstall");
    }

    //Target of Member plugin filter. Provides the plugin with Gravity Forms lists of capabilities
    public static function members_get_capabilities( $caps ) {
        return array_merge($caps, array("gravityforms_infusionsoft", "gravityforms_infusionsoft_uninstall"));
    }

    public static function disable_infusionsoft(){
        delete_option("gf_infusionsoft_settings");
    }
    
    public static function select_infusionsoft_form(){

        check_ajax_referer("gf_select_infusionsoft_form", "gf_select_infusionsoft_form");
        $form_id =  intval($_POST["form_id"]);
        $setting_id =  0; //intval($_POST["madmimi_setting_id"]);

        $valid = self::test_api();
        
        if(empty($valid)) {
            die("EndSelectForm();");
        }

        //getting list of all MadMimi merge variables for the selected contact list
        $merge_vars = self::get_fields();

        //getting configuration
        $config = GFInfusionsoftData::get_feed($setting_id);

        //getting field map UI
        $str = self::get_field_mapping($config, $form_id, $merge_vars);

        //fields meta
        $form = RGFormsModel::get_form_meta($form_id);
        //$fields = $form["fields"];
        die("EndSelectForm('" . str_replace("'", "\'", $str) . "', " . GFCommon::json_encode($form) . ");");
    }
    
    private static function get_fields() {
	    Infusionsoft_Classloader::loadClass('Contact');
		$Contact = new Infusionsoft_Contact();
		$fields = $Contact->getFields();
		
		foreach($fields as $key => $field) {
			
			$lists[] = array(
				'name' => esc_js($field),
				'req' => false, // (!empty($Field->createable) && empty($Field->nillable) && empty($Field->defaultedOnCreate)),
				'tag' => esc_js($field),
				#'type' => $Field->type,
				'length' => $Field->length,
			);
		}

		#asort($lists);
		
		return $lists;
    }
    
    private static function get_field_mapping($config = array(), $form_id, $merge_vars){

		$usedFields = array();
		$str = $custom = $standard = '';
		
        //getting list of all fields for the selected form
        $form_fields = self::get_form_fields($form_id);
        
        $str = "<table cellpadding='0' cellspacing='0'><thead><tr><th scope='col' class='infusionsoft_col_heading'>" . __("List Fields", "gravity-forms-infusionsoft") . "</th><th scope='col' class='infusionsoft_col_heading'>" . __("Form Fields", "gravity-forms-infusionsoft") . "</th></tr></thead><tbody>";
        
        foreach($merge_vars as $var){
        	
        	$selected_field = (isset($config["meta"]) && isset($config["meta"]["field_map"]) && isset($config["meta"]["field_map"][$var["tag"]])) ? $config["meta"]["field_map"][$var["tag"]] : '';
            $required = $var["req"] === true ? "<span class='gfield_required' title='This field is required.'>*</span>" : "";
            $error_class = $var["req"] === true && empty($selected_field) && !empty($_POST["gf_infusionsoft_submit"]) ? " feeds_validation_error" : "";
            $field_desc = '';
            $row = "<tr class='$error_class'><td class='infusionsoft_field_cell'><label for='infusionsoft_map_field_{$var['tag']}'>" . stripslashes( $var["name"] ) ." $required</label><small class='description' style='display:block'>{$field_desc}</small></td><td class='infusionsoft_field_cell'>" . self::get_mapped_field_list($var["tag"], $selected_field, $form_fields) . "</td></tr>";
            
            $str .= $row;

        } // End foreach merge var.
        
        $str .= "</tbody></table>";
        
        return $str;
    }
    
    private function getNewTag($tag, $used = array()) {
		if(isset($used[$tag])) {
			$i = 1;
			while($i < 1000) {
				if(!isset($used[$tag.'_'.$i])) {
					return $tag.'_'.$i;
				}
				$i++;
			}
		}
		return $tag;
    }
  	
    public static function get_form_fields($form_id){
        $form = RGFormsModel::get_form_meta($form_id);
        $fields = array();

        //Adding default fields
        array_push($form["fields"],array("id" => "date_created" , "label" => __("Entry Date", "gravity-forms-infusionsoft")));
        array_push($form["fields"],array("id" => "ip" , "label" => __("User IP", "gravity-forms-infusionsoft")));
        array_push($form["fields"],array("id" => "source_url" , "label" => __("Source Url", "gravity-forms-infusionsoft")));

        if(is_array($form["fields"])){
            foreach($form["fields"] as $field){
                if(isset($field["inputs"]) && is_array($field["inputs"]) && $field['type'] !== 'checkbox' && $field['type'] !== 'select'){

                    //If this is an address field, add full name to the list
                    if(RGFormsModel::get_input_type($field) == "address")
                        $fields[] =  array($field["id"], GFCommon::get_label($field) . " (" . __("Full" , "gravity-forms-infusionsoft") . ")");

                    foreach($field["inputs"] as $input)
                        $fields[] =  array($input["id"], GFCommon::get_label($field, $input["id"]));
                }
                else if(empty($field["displayOnly"])){
                    $fields[] =  array($field["id"], GFCommon::get_label($field));
                }
            }
        }
        return $fields;
    }

    private static function get_address($entry, $field_id){
        $street_value = str_replace("  ", " ", trim($entry[$field_id . ".1"]));
        $street2_value = str_replace("  ", " ", trim($entry[$field_id . ".2"]));
        $city_value = str_replace("  ", " ", trim($entry[$field_id . ".3"]));
        $state_value = str_replace("  ", " ", trim($entry[$field_id . ".4"]));
        $zip_value = trim($entry[$field_id . ".5"]);
        $country_value = GFCommon::get_country_code(trim($entry[$field_id . ".6"]));

        $address = $street_value;
        $address .= !empty($address) && !empty($street2_value) ? "  $street2_value" : $street2_value;
        $address .= !empty($address) && (!empty($city_value) || !empty($state_value)) ? "  $city_value" : $city_value;
        $address .= !empty($address) && !empty($city_value) && !empty($state_value) ? "  $state_value" : $state_value;
        $address .= !empty($address) && !empty($zip_value) ? "  $zip_value" : $zip_value;
        $address .= !empty($address) && !empty($country_value) ? "  $country_value" : $country_value;

        return $address;
    }

    public static function get_mapped_field_list($variable_name, $selected_field, $fields){
    	$field_name = "infusionsoft_map_field_" . $variable_name;
        $str = "<select name='$field_name' id='$field_name'><option value=''>" . __("", "gravity-forms-infusionsoft") . "</option>";
        foreach($fields as $field){
            $field_id = $field[0];
            $field_label = $field[1];
            $str .= "<option value='" . $field_id . "' ". selected(($field_id == $selected_field), true, false) . ">" . $field_label . "</option>";
        }
        $str .= "</select>";
        return $str;
    }
    
    public static function get_mapped_field_checkbox($variable_name, $selected_field, $field){
        $field_name = "infusionsoft_map_field_" . $variable_name;
        $field_id = $field[0];
        $str =  "<input name='$field_name' id='$field_name' type='checkbox' value='$field_id'";
        $selected = $field_id == $selected_field ? " checked='checked'" : false;
        if($selected) {
        	$str .= $selected; 
        }
    
        $str .= " />";
        return $str;
    }

    public static function export($entry, $form){
        //Login to Infusionsoft
        $api = self::get_api();
        if(!empty($api->lastError))
            return;

        //loading data class
        require_once(self::get_base_path() . "/data.php");

        //getting all active feeds
        $feeds = GFInfusionsoftData::get_feed_by_form($form["id"], true);
        foreach($feeds as $feed){
            //only export if user has opted in
            if(self::is_optin($form, $feed))
                self::export_feed($entry, $form, $feed, $api);
        }
    }

    public static function export_feed($entry, $form, $feed, $api){

        $double_optin = $feed["meta"]["double_optin"] ? true : false;
        $send_welcome = $feed["meta"]["welcome_email"] ? true : false;
        $email_field_id = $feed["meta"]["field_map"]["EMAIL"];
        $email = $entry[$email_field_id];

        $merge_vars = array();
        foreach($feed["meta"]["field_map"] as $var_tag => $field_id){

            $field = RGFormsModel::get_field($form, $field_id);
            if($field_id == intval($field_id) && RGFormsModel::get_input_type($field) == "address") //handling full address
                $merge_vars[$var_tag] = self::get_address($entry, $field_id);
            else if($var_tag != "EMAIL") //ignoring email field as it will be handled separatelly
                $merge_vars[$var_tag] = $entry[$field_id];
        }
        
        $valid = self::test_api();
        
       if($valid) {
	   		$retval = self::add_contact($email, $merge_vars);   
       } elseif(current_user_can('administrator')) {
	        echo '<div class="error" id="message">'.wpautop(sprintf(__("The form didn't create a contact because the Infusionsoft Gravity Forms Add-on plugin isn't properly configured. %sCheck the configuration%s and try again.", 'gravity-forms-infusionsoft'), '<a href="'.admin_url('admin.php?page=gf_settings&amp;addon=Infusionsoft').'">', '</a>')).'</div>';
	    }
    }
    
    private function add_contact($email_address, $merge_vars) {
        
        Infusionsoft_Classloader::loadClass('ContactService');
		
		$ContactService = new Infusionsoft_ContactService();
		
		// Options: 'Email', 'EmailAndName', 'EmailAndNameAndCompany'
		// http://help.infusionsoft.com/developers/services-methods/contacts/addWithDupCheck
		return $ContactService->addWithDupCheck($merge_vars, apply_filters('gravity_forms_infusionsoft_dupchecktype', 'Email'));
    }

    public static function uninstall(){

        //loading data lib
        require_once(self::get_base_path() . "/data.php");

        if(!GFInfusionsoft::has_access("gravityforms_infusionsoft_uninstall"))
            die(__("You don't have adequate permission to uninstall Infusionsoft Add-On.", "gravity-forms-infusionsoft"));

        //droping all tables
        GFInfusionsoftData::drop_tables();

        //removing options
        delete_option("gf_infusionsoft_settings");
        delete_option("gf_infusionsoft_version");

        //Deactivating plugin
        $plugin = "gravity-forms-infusionsoft/infusionsoft.php";
        deactivate_plugins($plugin);
        update_option('recently_activated', array($plugin => time()) + (array)get_option('recently_activated'));
    }

    public static function is_optin($form, $settings){
        $config = $settings["meta"];
        $operator = $config["optin_operator"];

        $field = RGFormsModel::get_field($form, $config["optin_field_id"]);
        $field_value = RGFormsModel::get_field_value($field, array());
        $is_value_match = is_array($field_value) ? in_array($config["optin_value"], $field_value) : $field_value == $config["optin_value"];

        return  !$config["optin_enabled"] || empty($field) || ($operator == "is" && $is_value_match) || ($operator == "isnot" && !$is_value_match);
    }


    private static function is_gravityforms_installed(){
        return class_exists("RGForms");
    }

    private static function is_gravityforms_supported(){
        if(class_exists("GFCommon")) {
            $is_correct_version = version_compare(GFCommon::$version, self::$min_gravityforms_version, ">=");
            return $is_correct_version;
        }
        else{
            return false;
        }
    }
    
  	private function simpleXMLToArray($xml,
                    $flattenValues=true,
                    $flattenAttributes = true,
                    $flattenChildren=true,
                    $valueKey='@value',
                    $attributesKey='@attributes',
                    $childrenKey='@children'){

        $return = array();
        if(!($xml instanceof SimpleXMLElement)){return $return;}
        $name = $xml->getName();
        $_value = trim((string)$xml);
        if(strlen($_value)==0){$_value = null;};

        if($_value!==null){
            if(!$flattenValues){$return[$valueKey] = $_value;}
            else{$return = $_value;}
        }

        $children = array();
        $first = true;
        foreach($xml->children() as $elementName => $child){
            $value = self::simpleXMLToArray($child, $flattenValues, $flattenAttributes, $flattenChildren, $valueKey, $attributesKey, $childrenKey);
            if(isset($children[$elementName])){
                if($first){
                    $temp = $children[$elementName];
                    unset($children[$elementName]);
                    $children[$elementName][] = $temp;
                    $first=false;
                }
                $children[$elementName][] = $value;
            }
            else{
                $children[$elementName] = $value;
            }
        }
        if(count($children)>0){
            if(!$flattenChildren){$return[$childrenKey] = $children;}
            else{$return = array_merge($return,$children);}
        }

        $attributes = array();
        foreach($xml->attributes() as $name=>$value){
            $attributes[$name] = trim($value);
        }
        if(count($attributes)>0){
            if(!$flattenAttributes){$return[$attributesKey] = $attributes;}
            else{$return = array_merge($return, $attributes);}
        }
        
        return $return;
    }
    
    private function convert_xml_to_object($response) {
  		$response = @simplexml_load_string($response);  // Added @ 1.2.2
		if(is_object($response)) {
		    return $response;
		} else {
		    return false;
		}
    }
    
    private function convert_xml_to_array($response) {
  		$response = self::convert_xml_to_object($response);
  		$response = self::simpleXMLToArray($response);
		if(is_array($response)) {
		    return $response;
		} else {
		    return false;
		}
    }

    protected static function has_access($required_permission){
        $has_members_plugin = function_exists('members_get_capabilities');
        $has_access = $has_members_plugin ? current_user_can($required_permission) : current_user_can("level_7");
        if($has_access)
            return $has_members_plugin ? $required_permission : "level_7";
        else
            return false;
    }

    //Returns the url of the plugin's root folder
    protected function get_base_url(){
        return plugins_url(null, __FILE__);
    }

    //Returns the physical path of the plugin's root folder
    protected function get_base_path(){
        $folder = basename(dirname(__FILE__));
        return WP_PLUGIN_DIR . "/" . $folder;
    }


}

?>